@@ -23,10 +23,6 @@
 #include <sys/stat.h>
 #include <mysql.h>
 
-#ifdef LATER_HAVE_NDBCLUSTER_DB
-#include "../ndb/src/mgmclient/ndb_mgmclient.h"
-#endif
-
 #define ADMIN_VERSION "8.42"
 #define MAX_MYSQL_VAR 256
 #define SHUTDOWN_DEF_TIMEOUT 3600		/* Wait for shutdown */
@@ -46,10 +42,6 @@ static uint tcp_port = 0, option_wait = 0, option_silent=0, nr_iterations;
 static uint opt_count_iterations= 0, my_end_arg;
 static ulong opt_connect_timeout, opt_shutdown_timeout;
 static char * unix_port=0;
-#ifdef LATER_HAVE_NDBCLUSTER_DB
-static my_bool opt_ndbcluster=0;
-static char *opt_ndb_connectstring=0;
-#endif
 
 #ifdef HAVE_SMEM
 static char *shared_memory_base_name=0;
@@ -105,9 +97,6 @@ enum commands {
   ADMIN_PING,             ADMIN_EXTENDED_STATUS, ADMIN_FLUSH_STATUS,
   ADMIN_FLUSH_PRIVILEGES, ADMIN_START_SLAVE,     ADMIN_STOP_SLAVE,
   ADMIN_FLUSH_THREADS,    ADMIN_OLD_PASSWORD
-#ifdef LATER_HAVE_NDBCLUSTER_DB
-  ,ADMIN_NDB_MGM
-#endif
 };
 static const char *command_names[]= {
   "create",               "drop",                "shutdown",
@@ -118,9 +107,6 @@ static const char *command_names[]= {
   "ping",                 "extended-status",     "flush-status",
   "flush-privileges",     "start-slave",         "stop-slave",
   "flush-threads","old-password",
-#ifdef LATER_HAVE_NDBCLUSTER_DB
-  "ndb-mgm",
-#endif
   NullS
 };
 
@@ -224,14 +210,6 @@ static struct my_option my_long_options[] =
   {"shutdown_timeout", OPT_SHUTDOWN_TIMEOUT, "", (uchar**) &opt_shutdown_timeout,
    (uchar**) &opt_shutdown_timeout, 0, GET_ULONG, REQUIRED_ARG,
    SHUTDOWN_DEF_TIMEOUT, 0, 3600*12, 0, 1, 0},
-#ifdef LATER_HAVE_NDBCLUSTER_DB
-  {"ndbcluster", OPT_NDBCLUSTER, ""
-   "", (uchar**) &opt_ndbcluster,
-   (uchar**) &opt_ndbcluster, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
-  {"ndb-connectstring", OPT_NDB_CONNECTSTRING, ""
-   "", (uchar**) &opt_ndb_connectstring,
-   (uchar**) &opt_ndb_connectstring, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-#endif
   { 0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}
 };
 
@@ -254,6 +232,8 @@ get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
     opt_count_iterations= 1;
     break;
   case 'p':
+    if (argument == disabled_my_option)
+      argument= (char*) "";			// Don't require password
     if (argument)
     {
       char *start=argument;
@@ -699,10 +679,16 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
 	pos=argv[1];
 	for (;;)
 	{
-	  if (mysql_kill(mysql,(ulong) atol(pos)))
+          /* We don't use mysql_kill(), since it only handles 32-bit IDs. */
+          char buff[26], *out; /* "KILL " + max 20 digs + NUL */
+          out= strxmov(buff, "KILL ", NullS);
+          ullstr(strtoull(pos, NULL, 0), out);
+
+          if (mysql_query(mysql, buff))
 	  {
-	    my_printf_error(0, "kill failed on %ld; error: '%s'", error_flags,
-			    atol(pos), mysql_error(mysql));
+            /* out still points to just the number */
+	    my_printf_error(0, "kill failed on %s; error: '%s'", error_flags,
+			    out, mysql_error(mysql));
 	    error=1;
 	  }
 	  if (!(pos=strchr(pos,',')))
@@ -859,7 +845,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
         bool old= (find_type(argv[0], &command_typelib, 2) ==
                    ADMIN_OLD_PASSWORD);
 #ifdef __WIN__
-        uint pw_len= strlen(pw);
+        uint pw_len= (uint) strlen(pw);
         if (pw_len > 1 && pw[0] == '\'' && pw[pw_len-1] == '\'')
           printf("Warning: single quotes were not trimmed from the password by"
                  " your command\nline client, as you might have expected.\n");
@@ -984,24 +970,6 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
       }
       mysql->reconnect=1;	/* Automatic reconnect is default */
       break;
-#ifdef LATER_HAVE_NDBCLUSTER_DB
-    case ADMIN_NDB_MGM:
-    {
-      if (argc < 2)
-      {
-	my_printf_error(0, "Too few arguments to ndb-mgm", error_flags);
-	return 1;
-      }
-      {
-        Ndb_mgmclient_handle cmd=
-	  ndb_mgmclient_handle_create(opt_ndb_connectstring);
-        ndb_mgmclient_execute(cmd, --argc, ++argv);
-	ndb_mgmclient_handle_destroy(cmd);
-      }
-      argc= 0;
-    }
-    break;
-#endif
     default:
       my_printf_error(0, "Unknown command: '%-.60s'", error_flags, argv[0]);
       return 1;
