@@ -712,6 +712,8 @@ get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
     verbose++;
     break;
   case 'p':
+    if (argument == disabled_my_option)
+      argument= (char*) "";			/* Don't require password */
     if (argument)
     {
       char *start= argument;
@@ -1929,7 +1931,7 @@ parse_option(const char *origin, option_string **stmt, char delm)
   char *ptr= (char *)origin;
   option_string **sptr= stmt;
   option_string *tmp;
-  uint length= strlen(origin);
+  size_t length= strlen(origin);
   uint count= 0; /* We know that there is always one */
 
   for (tmp= *sptr= (option_string *)my_malloc(sizeof(option_string),
