@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -36,22 +35,19 @@ static SECURITY_ATTRIBUTES mmap_security_attributes=
 void *my_mmap(void *addr, size_t len, int prot,
                int flags, int fd, my_off_t offset)
 {
-  DWORD flProtect=0;
   HANDLE hFileMap;
   LPVOID ptr;
   HANDLE hFile= (HANDLE)_get_osfhandle(fd);
   if (hFile == INVALID_HANDLE_VALUE)
     return MAP_FAILED;
 
-  flProtect|=SEC_COMMIT;
-
   hFileMap=CreateFileMapping(hFile, &mmap_security_attributes,
                              PAGE_READWRITE, 0, (DWORD) len, NULL);
   if (hFileMap == 0)
     return MAP_FAILED;
 
   ptr=MapViewOfFile(hFileMap,
-                    flags & PROT_WRITE ? FILE_MAP_WRITE : FILE_MAP_READ,
+                    prot & PROT_WRITE ? FILE_MAP_WRITE : FILE_MAP_READ,
                     (DWORD)(offset >> 32), (DWORD)offset, len);
 
   /*
