@@ -30,7 +30,7 @@ pthread_key(struct st_my_thread_var, THR_KEY_mysys);
 #endif /* USE_TLS */
 pthread_mutex_t THR_LOCK_malloc,THR_LOCK_open,
 	        THR_LOCK_lock,THR_LOCK_isam,THR_LOCK_myisam,THR_LOCK_heap,
-                THR_LOCK_net, THR_LOCK_charset, THR_LOCK_threads;
+                THR_LOCK_net, THR_LOCK_charset, THR_LOCK_threads, THR_LOCK_time;
 pthread_cond_t  THR_COND_threads;
 uint            THR_thread_count= 0;
 uint 		my_thread_end_wait_time= 5;
@@ -50,12 +50,12 @@ pthread_mutexattr_t my_errorcheck_mutexattr;
 #ifdef TARGET_OS_LINUX
 
 /*
- Dummy thread spawned in my_thread_global_init() below to avoid
- race conditions in NPTL pthread_exit code.
+  Dummy thread spawned in my_thread_global_init() below to avoid
+  race conditions in NPTL pthread_exit code.
 */
 
 static pthread_handler_t
-nptl_pthread_exit_hack_handler(void *arg __attribute__((unused)))
+nptl_pthread_exit_hack_handler(void *arg __attribute((unused)))
 {
   /* Do nothing! */
   pthread_exit(0);
@@ -64,6 +64,7 @@ nptl_pthread_exit_hack_handler(void *arg __attribute__((unused)))
 
 #endif /* TARGET_OS_LINUX */
 
+
 static uint get_thread_lib(void);
 
 /*
@@ -146,6 +147,7 @@ my_bool my_thread_global_init(void)
   pthread_mutex_init(&THR_LOCK_net,MY_MUTEX_INIT_FAST);
   pthread_mutex_init(&THR_LOCK_charset,MY_MUTEX_INIT_FAST);
   pthread_mutex_init(&THR_LOCK_threads,MY_MUTEX_INIT_FAST);
+  pthread_mutex_init(&THR_LOCK_time,MY_MUTEX_INIT_FAST);
   pthread_cond_init(&THR_COND_threads, NULL);
 #if defined( __WIN__) || defined(OS2)
   win_pthread_init();
@@ -209,6 +211,7 @@ void my_thread_global_end(void)
   pthread_mutex_destroy(&THR_LOCK_myisam);
   pthread_mutex_destroy(&THR_LOCK_heap);
   pthread_mutex_destroy(&THR_LOCK_net);
+  pthread_mutex_destroy(&THR_LOCK_time);
   pthread_mutex_destroy(&THR_LOCK_charset);
   if (all_threads_killed)
   {
@@ -223,7 +226,7 @@ void my_thread_global_end(void)
 #endif
 }
 
-static long thread_id=0;
+static my_thread_id thread_id= 0;
 
 /*
   Allocate thread specific memory for the thread, used by mysys and dbug
@@ -251,15 +254,16 @@ my_bool my_thread_init(void)
   my_bool error=0;
 
 #ifdef EXTRA_DEBUG_THREADS
-  fprintf(stderr,"my_thread_init(): thread_id=%ld\n",pthread_self());
+  fprintf(stderr,"my_thread_init(): thread_id: 0x%lx\n",
+          (ulong) pthread_self());
 #endif  
 
 #if !defined(__WIN__) || defined(USE_TLS)
   if (my_pthread_getspecific(struct st_my_thread_var *,THR_KEY_mysys))
   {
 #ifdef EXTRA_DEBUG_THREADS
-    fprintf(stderr,"my_thread_init() called more than once in thread %ld\n",
-	        pthread_self());
+    fprintf(stderr,"my_thread_init() called more than once in thread 0x%lx\n",
+            (long) pthread_self());
 #endif    
     goto end;
   }
@@ -279,7 +283,9 @@ my_bool my_thread_init(void)
   tmp= &THR_KEY_mysys;
 #endif
 #if defined(__WIN__) && defined(EMBEDDED_LIBRARY)
-  tmp->thread_self= (pthread_t)getpid();
+  tmp->pthread_self= (pthread_t) getpid();
+#else
+  tmp->pthread_self= pthread_self();
 #endif
   pthread_mutex_init(&tmp->mutex,MY_MUTEX_INIT_FAST);
   pthread_cond_init(&tmp->suspend, NULL);
@@ -289,6 +295,11 @@ my_bool my_thread_init(void)
   tmp->id= ++thread_id;
   ++THR_thread_count;
   pthread_mutex_unlock(&THR_LOCK_threads);
+#ifndef DBUG_OFF
+  /* Generate unique name for thread */
+  (void) my_thread_name();
+#endif
+
 end:
   return error;
 }
@@ -312,8 +323,8 @@ void my_thread_end(void)
   tmp= my_pthread_getspecific(struct st_my_thread_var*,THR_KEY_mysys);
 
 #ifdef EXTRA_DEBUG_THREADS
-  fprintf(stderr,"my_thread_end(): tmp: 0x%lx  thread_id=%ld\n",
-	  (long) tmp, pthread_self());
+  fprintf(stderr,"my_thread_end(): tmp: 0x%lx  pthread_self: 0x%lx  thread_id: %ld\n",
+	  (long) tmp, (long) pthread_self(), tmp ? (long) tmp->id : 0L);
 #endif  
   if (tmp && tmp->init)
   {
@@ -321,16 +332,17 @@ void my_thread_end(void)
     /* tmp->dbug is allocated inside DBUG library */
     if (tmp->dbug)
     {
+      DBUG_POP();
       free(tmp->dbug);
       tmp->dbug=0;
     }
 #endif
-#if !defined(__bsdi__) && !defined(__OpenBSD__) || defined(HAVE_mit_thread)
+#if !defined(__bsdi__) && !defined(__OpenBSD__)
  /* bsdi and openbsd 3.5 dumps core here */
     pthread_cond_destroy(&tmp->suspend);
 #endif
     pthread_mutex_destroy(&tmp->mutex);
-#if (!defined(__WIN__) && !defined(OS2)) || defined(USE_TLS)
+#if !defined(__WIN__) || defined(USE_TLS)
     free(tmp);
 #else
     tmp->init= 0;
@@ -349,24 +361,14 @@ void my_thread_end(void)
    pthread_mutex_unlock(&THR_LOCK_threads);
   }
   /* The following free has to be done, even if my_thread_var() is 0 */
-#if (!defined(__WIN__) && !defined(OS2)) || defined(USE_TLS)
+#if !defined(__WIN__) || defined(USE_TLS)
   pthread_setspecific(THR_KEY_mysys,0);
 #endif
 }
 
 struct st_my_thread_var *_my_thread_var(void)
 {
-  struct st_my_thread_var *tmp=
-    my_pthread_getspecific(struct st_my_thread_var*,THR_KEY_mysys);
-#if defined(USE_TLS)
-  /* This can only happen in a .DLL */
-  if (!tmp)
-  {
-    my_thread_init();
-    tmp=my_pthread_getspecific(struct st_my_thread_var*,THR_KEY_mysys);
-  }
-#endif
-  return tmp;
+  return  my_pthread_getspecific(struct st_my_thread_var*,THR_KEY_mysys);
 }
 
 
@@ -374,17 +376,9 @@ struct st_my_thread_var *_my_thread_var(void)
   Get name of current thread.
 ****************************************************************************/
 
-#define UNKNOWN_THREAD -1
-
-long my_thread_id()
+my_thread_id my_thread_dbug_id()
 {
-#if defined(HAVE_PTHREAD_GETSEQUENCE_NP)
-  return pthread_getsequence_np(pthread_self());
-#elif (defined(__sun) || defined(__sgi) || defined(__linux__)) && !defined(HAVE_mit_thread)
-  return pthread_self();
-#else
   return my_thread_var->id;
-#endif
 }
 
 #ifdef DBUG_OFF
@@ -401,8 +395,8 @@ const char *my_thread_name(void)
   struct st_my_thread_var *tmp=my_thread_var;
   if (!tmp->name[0])
   {
-    long id=my_thread_id();
-    sprintf(name_buff,"T@%ld", id);
+    my_thread_id id= my_thread_dbug_id();
+    sprintf(name_buff,"T@%lu", (ulong) id);
     strmake(tmp->name,name_buff,THREAD_NAME_SIZE);
   }
   return tmp->name;
