@@ -20,6 +20,30 @@
 #include <m_ctype.h>
 
 
+static const char field_separator=',';
+
+int find_type_or_exit(const char *x, TYPELIB *typelib, const char *option)
+{
+  int res;
+  const char **ptr;
+
+  if ((res= find_type((char *) x, typelib, 2)) <= 0)
+  {
+    ptr= typelib->type_names;
+    if (!*x)
+      fprintf(stderr, "No option given to %s\n", option);
+    else
+      fprintf(stderr, "Unknown option to %s: %s\n", option, x);
+    fprintf(stderr, "Alternatives are: '%s'", *ptr);
+    while (*++ptr)
+      fprintf(stderr, ",'%s'", *ptr);
+    fprintf(stderr, "\n");
+    exit(1);
+  }
+  return res;
+}
+
+
 /*
   Search after a string in a list of strings. Endspace in x is not compared.
 
@@ -31,6 +55,7 @@
 			If & 1 accept only whole names
 			If & 2 don't expand if half field
 			If & 4 allow #number# as type
+			If & 8 use ',' as string terminator
 
   NOTES
     If part, uniq field is found and full_name == 0 then x is expanded
@@ -42,10 +67,11 @@
     >0  Offset+1 in typelib for matched string
 */
 
-int find_type(my_string x, TYPELIB *typelib, uint full_name)
+
+int find_type(char *x, const TYPELIB *typelib, uint full_name)
 {
   int find,pos,findpos;
-  reg1 my_string i;
+  reg1 char * i;
   reg2 const char *j;
   DBUG_ENTER("find_type");
   DBUG_PRINT("enter",("x: '%s'  lib: 0x%lx", x, (long) typelib));
@@ -60,16 +86,18 @@ int find_type(my_string x, TYPELIB *typelib, uint full_name)
   for (pos=0 ; (j=typelib->type_names[pos]) ; pos++)
   {
     for (i=x ; 
-    	*i && my_toupper(&my_charset_latin1,*i) == 
+    	*i && (!(full_name & 8) || *i != field_separator) &&
+        my_toupper(&my_charset_latin1,*i) == 
     		my_toupper(&my_charset_latin1,*j) ; i++, j++) ;
     if (! *j)
     {
       while (*i == ' ')
 	i++;					/* skip_end_space */
-      if (! *i)
+      if (! *i || ((full_name & 8) && *i == field_separator))
 	DBUG_RETURN(pos+1);
     }
-    if (! *i && (!*j || !(full_name & 1)))
+    if ((!*i && (!(full_name & 8) || *i != field_separator)) && 
+        (!*j || !(full_name & 1)))
     {
       find++;
       findpos=pos;
@@ -97,7 +125,7 @@ int find_type(my_string x, TYPELIB *typelib, uint full_name)
 	/* Get name of type nr 'nr' */
 	/* Warning first type is 1, 0 = empty field */
 
-void make_type(register my_string to, register uint nr,
+void make_type(register char * to, register uint nr,
 	       register TYPELIB *typelib)
 {
   DBUG_ENTER("make_type");
@@ -120,6 +148,50 @@ const char *get_type(TYPELIB *typelib, uint nr)
 }
 
 
+/*
+  Create an integer value to represent the supplied comma-seperated
+  string where each string in the TYPELIB denotes a bit position.
+
+  SYNOPSIS
+    find_typeset()
+    x		string to decompose
+    lib		TYPELIB (struct of pointer to values + count)
+    err		index (not char position) of string element which was not 
+                found or 0 if there was no error
+
+  RETURN
+    a integer representation of the supplied string
+*/
+
+my_ulonglong find_typeset(char *x, TYPELIB *lib, int *err)
+{
+  my_ulonglong result;
+  int find;
+  char *i;
+  DBUG_ENTER("find_set");
+  DBUG_PRINT("enter",("x: '%s'  lib: 0x%lx", x, (long) lib));
+
+  if (!lib->count)
+  {
+    DBUG_PRINT("exit",("no count"));
+    DBUG_RETURN(0);
+  }
+  result= 0;
+  *err= 0;
+  while (*x)
+  {
+    (*err)++;
+    i= x;
+    while (*x && *x != field_separator) x++;
+    if ((find= find_type(i, lib, 2 | 8) - 1) < 0)
+      DBUG_RETURN(0);
+    result|= (ULL(1) << find);
+  }
+  *err= 0;
+  DBUG_RETURN(result);
+} /* find_set */
+
+
 /*
   Create a copy of a specified TYPELIB structure.
 
