@@ -1,5 +1,4 @@
-/* Copyright (c) 2001-2008 MySQL AB, 2008, 2009 Sun Microsystems, Inc.
-   Use is subject to license terms.
+/* Copyright 2000-2008 MySQL AB, 2008 Sun Microsystems, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 
 /*
@@ -59,13 +58,13 @@ bool select_union::send_data(List<Item> &values)
     return 0;
   }
   fill_record(thd, table->field, values, 1);
-  if (thd->net.report_error)
+  if (thd->is_error())
     return 1;
 
-  if ((error= table->file->write_row(table->record[0])))
+  if ((error= table->file->ha_write_row(table->record[0])))
   {
     /* create_myisam_from_heap will generate error if needed */
-    if (error != HA_ERR_FOUND_DUPP_KEY && error != HA_ERR_FOUND_DUPP_UNIQUE &&
+    if (table->file->is_fatal_error(error, HA_CHECK_DUP) &&
         create_myisam_from_heap(thd, table, &tmp_table_param, error, 1))
       return 1;
   }
@@ -145,8 +144,8 @@ void
 st_select_lex_unit::init_prepare_fake_select_lex(THD *thd_arg) 
 {
   thd_arg->lex->current_select= fake_select_lex;
-  fake_select_lex->table_list.link_in_list((byte *)&result_table_list,
-					   (byte **)
+  fake_select_lex->table_list.link_in_list((uchar *)&result_table_list,
+					   (uchar **)
 					   &result_table_list.next_local);
   fake_select_lex->context.table_list= 
     fake_select_lex->context.first_name_resolution_table= 
@@ -162,8 +161,8 @@ st_select_lex_unit::init_prepare_fake_select_lex(THD *thd_arg)
        order;
        order=order->next)
   {
-    (*order->item)->walk(&Item::change_context_processor,
-                         (byte *) &fake_select_lex->context);
+    (*order->item)->walk(&Item::change_context_processor, 0,
+                         (uchar*) &fake_select_lex->context);
   }
 }
 
@@ -174,7 +173,7 @@ bool st_select_lex_unit::prepare(THD *thd_arg, select_result *sel_result,
   SELECT_LEX *lex_select_save= thd_arg->lex->current_select;
   SELECT_LEX *sl, *first_sl= first_select();
   select_result *tmp_result;
-  bool is_union;
+  bool is_union_select;
   TABLE *empty_table= 0;
   DBUG_ENTER("st_select_lex_unit::prepare");
 
@@ -212,11 +211,11 @@ bool st_select_lex_unit::prepare(THD *thd_arg, select_result *sel_result,
   
   thd_arg->lex->current_select= sl= first_sl;
   found_rows_for_union= first_sl->options & OPTION_FOUND_ROWS;
-  is_union= first_sl->next_select() || fake_select_lex;
+  is_union_select= is_union() || fake_select_lex;
 
   /* Global option */
 
-  if (is_union)
+  if (is_union_select)
   {
     if (!(tmp_result= union_result= new select_union))
       goto err;
@@ -247,7 +246,7 @@ bool st_select_lex_unit::prepare(THD *thd_arg, select_result *sel_result,
 
     thd_arg->lex->current_select= sl;
 
-    can_skip_order_by= is_union && !(sl->braces && sl->explicit_limit);
+    can_skip_order_by= is_union_select && !(sl->braces && sl->explicit_limit);
 
     saved_error= join->prepare(&sl->ref_pointer_array,
                                (TABLE_LIST*) sl->table_list.first,
@@ -260,7 +259,7 @@ bool st_select_lex_unit::prepare(THD *thd_arg, select_result *sel_result,
                                (ORDER*) 0 : (ORDER *)sl->order_list.first,
                                (ORDER*) sl->group_list.first,
                                sl->having,
-                               (is_union ? (ORDER*) 0 :
+                               (is_union_select ? (ORDER*) 0 :
                                 (ORDER*) thd_arg->lex->proc_list.first),
                                sl, this);
     /* There are no * in the statement anymore (for PS) */
@@ -273,7 +272,7 @@ bool st_select_lex_unit::prepare(THD *thd_arg, select_result *sel_result,
       Use items list of underlaid select for derived tables to preserve
       information about fields lengths and exact types
     */
-    if (!is_union)
+    if (!is_union_select)
       types= first_sl->item_list;
     else if (sl == first_sl)
     {
@@ -316,7 +315,7 @@ bool st_select_lex_unit::prepare(THD *thd_arg, select_result *sel_result,
     }
   }
 
-  if (is_union)
+  if (is_union_select)
   {
     /*
       Check that it was possible to aggregate
@@ -440,10 +439,10 @@ bool st_select_lex_unit::exec()
       {
         item->assigned(0); // We will reinit & rexecute unit
         item->reset();
-        table->file->delete_all_rows();
+        table->file->ha_delete_all_rows();
       }
       /* re-enabling indexes for next subselect iteration */
-      if (union_distinct && table->file->enable_indexes(HA_KEY_SWITCH_ALL))
+      if (union_distinct && table->file->ha_enable_indexes(HA_KEY_SWITCH_ALL))
       {
         DBUG_ASSERT(0);
       }
@@ -481,11 +480,11 @@ bool st_select_lex_unit::exec()
       }
       if (!saved_error)
       {
-	records_at_start= table->file->records;
+	records_at_start= table->file->stats.records;
 	sl->join->exec();
         if (sl == union_distinct)
 	{
-	  if (table->file->disable_indexes(HA_KEY_SWITCH_ALL))
+	  if (table->file->ha_disable_indexes(HA_KEY_SWITCH_ALL))
 	    DBUG_RETURN(TRUE);
 	  table->no_keyread=1;
 	}
@@ -525,7 +524,7 @@ bool st_select_lex_unit::exec()
 	  rows and actual rows added to the temporary table.
 	*/
 	add_rows+= (ulonglong) (thd->limit_found_rows - (ulonglong)
-			      ((table->file->records -  records_at_start)));
+			      ((table->file->stats.records -  records_at_start)));
       }
     }
   }
@@ -608,7 +607,7 @@ bool st_select_lex_unit::exec()
       fake_select_lex->table_list.empty();
       if (!saved_error)
       {
-	thd->limit_found_rows = (ulonglong)table->file->records + add_rows;
+	thd->limit_found_rows = (ulonglong)table->file->stats.records + add_rows;
         thd->examined_row_count+= examined_rows;
       }
       /*
@@ -682,7 +681,7 @@ void st_select_lex_unit::reinit_exec_mechanism()
 {
   prepared= optimized= executed= 0;
 #ifndef DBUG_OFF
-  if (first_select()->next_select())
+  if (is_union())
   {
     List_iterator_fast<Item> it(item_list);
     Item *field;
@@ -749,7 +748,6 @@ bool st_select_lex_unit::change_result(select_subselect *new_result,
 List<Item> *st_select_lex_unit::get_unit_column_types()
 {
   SELECT_LEX *sl= first_select();
-  bool is_union= test(sl->next_select());
   bool is_procedure= test(sl->join->procedure);
 
   if (is_procedure)
@@ -760,7 +758,7 @@ List<Item> *st_select_lex_unit::get_unit_column_types()
   }
 
 
-  if (is_union)
+  if (is_union())
   {
     DBUG_ASSERT(prepared);
     /* Types are generated during prepare */
