@@ -256,6 +256,12 @@ ulint srv_read_ahead_seq = 0;
 /* variable to count the number of random read-aheads */
 ulint srv_read_ahead_rnd = 0;
 
+/* An option to enable the fix for "Bug#43660 SHOW INDEXES/ANALYZE does
+NOT update cardinality for indexes of InnoDB table". By default we are
+running with the fix disabled because MySQL 5.1 is frozen for such
+behavioral changes. */
+char srv_use_legacy_cardinality_algorithm = TRUE;
+
 /* structure to pass status variables to MySQL */
 export_struc export_vars;
 
@@ -279,17 +285,19 @@ computer. Bigger computers need bigger values. Value 0 will disable the
 concurrency check. */
 
 ulong	srv_thread_concurrency	= 0;
-ulong	srv_commit_concurrency	= 0;
 
 os_fast_mutex_t	srv_conc_mutex;		/* this mutex protects srv_conc data
 					structures */
-lint	srv_conc_n_threads	= 0;	/* number of OS threads currently
-					inside InnoDB; it is not an error
-					if this drops temporarily below zero
-					because we do not demand that every
-					thread increments this, but a thread
-					waiting for a lock decrements this
-					temporarily */
+lint	srv_conc_n_threads	= 0;	/* number of transactions that
+					have declared_to_be_inside_innodb
+					set. It used to be a non-error
+					for this value to drop below
+					zero temporarily. This is no
+					longer true. We'll, however,
+					keep the lint datatype to add
+					assertions to catch any corner
+					cases that we may have
+					missed. */
 ulint	srv_conc_n_waiting_threads = 0;	/* number of OS threads waiting in the
 					FIFO for a permission to enter InnoDB
 					*/
@@ -328,8 +336,6 @@ ulint	srv_fast_shutdown	= 0;
 /* Generate a innodb_status.<pid> file */
 ibool	srv_innodb_status	= FALSE;
 
-ibool	srv_stats_on_metadata	= TRUE;
-
 ibool	srv_use_doublewrite_buf	= TRUE;
 ibool	srv_use_checksums = TRUE;
 
@@ -1022,6 +1028,8 @@ retry:
 		return;
 	}
 
+	ut_ad(srv_conc_n_threads >= 0);
+
 	if (srv_conc_n_threads < (lint)srv_thread_concurrency) {
 
 		srv_conc_n_threads++;
@@ -1148,6 +1156,8 @@ srv_conc_force_enter_innodb(
 		return;
 	}
 
+	ut_ad(srv_conc_n_threads >= 0);
+
 	os_fast_mutex_lock(&srv_conc_mutex);
 
 	srv_conc_n_threads++;
@@ -1169,11 +1179,6 @@ srv_conc_force_exit_innodb(
 {
 	srv_conc_slot_t*	slot	= NULL;
 
-	if (UNIV_LIKELY(!srv_thread_concurrency)) {
-
-		return;
-	}
-
 	if (trx->mysql_thd != NULL
 	    && thd_is_replication_slave_thread(trx->mysql_thd)) {
 
@@ -1187,6 +1192,7 @@ srv_conc_force_exit_innodb(
 
 	os_fast_mutex_lock(&srv_conc_mutex);
 
+	ut_ad(srv_conc_n_threads > 0);
 	srv_conc_n_threads--;
 	trx->declared_to_be_inside_innodb = FALSE;
 	trx->n_tickets_to_enter_innodb = 0;
